import { BaseBatchDeleteManager } from 'src/core/modules/domain/usecase/managers/base-batch-delete.manager';
import { ItemEntity } from '../../entities/item.entity';
import {
  EventTopics,
  validateRelations,
} from 'src/core/strings/constants/interface.constants';
import { ItemModel } from '../../../data/models/item.model';
import { ItemDeletedEvent } from '../../entities/event/item-deleted.event';
import { BatchResult } from 'src/core/response/domain/ok-response.interface';
import {
  HttpStatus,
  Injectable,
  UnprocessableEntityException,
} from '@nestjs/common';

@Injectable()
export class BatchDeleteItemManager extends BaseBatchDeleteManager<ItemEntity> {
  async beforeProcess(): Promise<void> {
    return;
  }

  async validateData(data: ItemEntity): Promise<void> {
    const haveRelation = await this.dataService.getOneByOptions({
      where: {
        bundling_items: {
          id: data.id,
        },
      },
    });

    if (haveRelation) {
      throw new UnprocessableEntityException({
        statusCode: HttpStatus.UNPROCESSABLE_ENTITY,
        message: `Failed! this data already connected to bunding item`,
        error: 'Unprocessable Entity',
      });
    }
    return;
  }

  async afterProcess(): Promise<void> {
    return;
  }

  get validateRelations(): validateRelations[] {
    return [];
  }

  get entityTarget(): any {
    return ItemModel;
  }

  get eventTopics(): EventTopics[] {
    return [
      {
        topic: ItemDeletedEvent,
      },
    ];
  }

  getResult(): BatchResult {
    return this.result;
  }
}
