import {
  HttpStatus,
  Injectable,
  UnprocessableEntityException,
} from '@nestjs/common';
import { BaseDeleteManager } from 'src/core/modules/domain/usecase/managers/base-delete.manager';
import { ItemEntity } from '../../entities/item.entity';
import {
  EventTopics,
  validateRelations,
} from 'src/core/strings/constants/interface.constants';
import { ItemModel } from '../../../data/models/item.model';
import { ItemDeletedEvent } from '../../entities/event/item-deleted.event';

@Injectable()
export class DeleteItemManager extends BaseDeleteManager<ItemEntity> {
  getResult(): string {
    return `Success`;
  }

  async validateProcess(): Promise<void> {
    const haveRelation = await this.dataService.getOneByOptions({
      where: {
        bundling_items: {
          id: this.dataId,
        },
      },
    });

    if (haveRelation) {
      throw new UnprocessableEntityException({
        statusCode: HttpStatus.UNPROCESSABLE_ENTITY,
        message: `Failed! this data already connected to bunding item`,
        error: 'Unprocessable Entity',
      });
    }
    return;
  }

  async beforeProcess(): Promise<void> {
    return;
  }

  async afterProcess(): Promise<void> {
    return;
  }

  get validateRelations(): validateRelations[] {
    return [];
  }

  get entityTarget(): any {
    return ItemModel;
  }

  get eventTopics(): EventTopics[] {
    return [
      {
        topic: ItemDeletedEvent,
        data: this.data,
      },
    ];
  }
}
